/**
 * This tries to duplicate the neat Eclipse editing feature where you can rename something
 * in place, ie you select som etext, type control-1 and start typing all occurences of the selected
 * text change in place at the same time.
 */
#include "eel.h"        /* standard definitions */

int rename_in_place_active= 0;

typedef struct {
	spot s1, s2;
} t_rename_spots;

zeroed buffer t_rename_spots rename_source;
t_rename_spots *rename_points;
int rename_points_size= 0;
int rename_points_cnt= 0;

int inside_rename()
{
	if(point >= *rename_source.s1 && point <= *rename_source.s2)
		return 1;

	return 0;
}

int rename_when_idle(int cnt); // forward reference

// deactivate rename and release its resources
int deactivate_rename()
{
	int i;
	rename_in_place_active= 0;

	highlight_off();	

	// release allocations
	for(i=0;i<rename_points_cnt;i++){
		remove_region(i+2000);
		free_spot(rename_points[i].s1);
		free_spot(rename_points[i].s2);
	}
	free(rename_points);
	rename_points_size= 0;
	rename_points_cnt= 0;

	free_spot(rename_source.s1);
	free_spot(rename_source.s2);

	delete_buffer_when_idle(bufnum, rename_when_idle);
}   	

void new_rename_in_place_when_aborting()
{
	if(rename_in_place_active == 1){
		deactivate_rename();
	}

	rename_in_place_when_aborting();
}

REPLACE_FUNC("rename_in_place", "when_aborting")

// This routine is used to mirror the changes, check if we are still inside the rename region
int rename_when_idle(int cnt)
{
	int i;
	if(rename_in_place_active == 0) return -1;

	// if we moved point out of rename disable rename
	if(!inside_rename()){
		deactivate_rename();
		return -1;
	}

	for(i=0;i<rename_points_cnt;i++){
		// get text from the rename region and overwrite this text with that
		char *buf1= NULL, *buf2= NULL;
		grab_expanding(*rename_source.s1, *rename_source.s2, &buf1, 10);
		grab_expanding(*rename_points[i].s1, *rename_points[i].s2, &buf2, 10);
		if(strcmp(buf1, buf2) != 0){ // only if different
			save_spot point;
			save_spot mark;
			int hon= is_highlight_on();
			int p1= *rename_points[i].s1;
			int p2= *rename_points[i].s2;
			delete(p1, p2);
			point= *rename_points[i].s1;
			stuff(buf1);
			if(hon) highlight_on();
		}
		free(buf1);
		free(buf2);
	}
	return -1;
}

color_class ident_completion;
color_class rename_in_place_highlight = color_class ident_completion;

command rename_in_place()  on reg_tab[NUMCTRL('1')]
{
	// first get highlighted text
	char *tag;
	fix_region();
	if(is_highlight_on() && mark - point > 0){
		tag= malloc((mark-point)+1);
		grab(point, mark, tag);
	}else{
		error("Nothing selected");
	}

	// remember spot positions
	rename_source.s1= alloc_spot(0);
	set_spot(&rename_source.s1, point);

	rename_source.s2= alloc_spot(1);
	set_spot(&rename_source.s2, mark);

	// next find all matching occurences and highlight them too
	point= mark;
	int cnt= 0;
	int i;
	while(search(1, tag)){
		if(cnt >= rename_points_size){
			t_rename_spots *ts= (t_rename_spots *)malloc(sizeof(t_rename_spots)*(rename_points_size + 10));
			if(rename_points_size > 0){
				for(i=0;i<rename_points_size;i++)
					ts[i]= rename_points[i];
				free(rename_points);
			}
			rename_points= ts;
			rename_points_size += 10;
		}

		rename_points[cnt].s1= alloc_spot(0);
		rename_points[cnt].s2= alloc_spot(1);
		set_spot(&rename_points[cnt].s1, matchstart);
		set_spot(&rename_points[cnt].s2, matchend);
		add_region(rename_points[cnt].s1, rename_points[cnt].s2, color_class rename_in_place_highlight, REGNORM, cnt+2000);
		cnt++;
	}
	rename_points_cnt= cnt;
	point= *rename_source.s1;

	// turn on when_idle
	if(rename_points_cnt > 0){
		add_buffer_when_idle(bufnum, rename_when_idle);	
		rename_in_place_active= 1;
	}

	free(tag);
}
